﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ExpressionEditor.Sample
{
    public partial class FrmCompileListSample : Form
    {
        private static string _path;
        //private CompileCond _curCompileCond;
        private DataTable _table;

        public FrmCompileListSample()
        {
            InitializeComponent();
        }

        private void FrmMath_Load(object sender, EventArgs e)
        {
            _table = new DataTable();
            _table.Columns.Add("模板名称");
            _table.Columns.Add("地址");
            foreach (var item in BinarySerialize.GetConds())
            {
                _table.Rows.Add(Path.GetFileNameWithoutExtension(item), item);
            }
            dataGridView1.AllowUserToAddRows = false;
            dataGridView1.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;
            dataGridView1.DataSource = _table;
            _path = BinarySerialize.GetPath(this.GetType());
            cmbTimes.SelectedIndex = 0;
            txtResult.Text = "计算公式已经在内部设定，可以直接计算！";
            //if (File.Exists(_path))
            //{
            //    _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
            //}
        }

        private void AddUseExpr(CompileCond cc)
        {
            cc.VarUserExprs.Clear();
            cc.VarUserExprs.Add(new UserExpr()
            {
                Catalog = "Self",
                Syntax = "_SayHelloWorld(string name, string world)",
                Description = "一个给大家Say Hello 的方法",
                Example = "string _str = _SayHelloWorld(\"张三\",\"大家好\")",
                Name = "_SayHelloWorld",
                Function = new Func<string, string, string>(this._SayHelloWorld),
            });
            cc.VarUserExprs.Add(new UserExpr()
            {
                Catalog = "Self",
                Syntax = "_小明哥哥",
                Description = "小明哥哥属性",
                Example = "string _str = _小明哥哥",
                Name = "_小明哥哥",
                ReturnType = typeof(string).FullName,
                Content = "\"大明\" + DateTime.Now.ToString(\"MMdd\")",
                IsMethod = false
            });
            cc.VarUserExprs.Add(new UserExpr()
            {
                Catalog = "Self",
                Syntax = "_GetAge(string name)",
                Description = "获取年龄",
                Example = "int _age = _GetAge(\"张三\")",
                Name = "_GetAge(string name)",
                ReturnType = typeof(int).FullName,
                Content = "return DateTime.Now.Year + 18;",
                IsMethod = true
            });
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="name"></param>
        /// <param name="world"></param>
        /// <returns></returns>
        private string _SayHelloWorld(string name, string world)
        {
            return name + "讲 : " + world + " 当前时间为： " + DateTime.Now;
        }

        private void tsbExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void tsbCalc_Click(object sender, EventArgs e)
        {
            CompileCond _cc = new CompileCond();
            _cc.Name = this.GetType().Name;
            var _table = new DataTable();
            _table.Columns.Add("物品");
            _table.Columns.Add("单价", typeof(Decimal));
            _table.Columns.Add("数量", typeof(Decimal));
            _table.Rows.Add("钢笔", 10, 2);
            _table.Rows.Add("硬皮抄", 10, 3);
            _table.Rows.Add("沐浴露", 20, 1);
            _cc.VarTable = _table;
            _cc.VarList.Add(new VariableItem("打折", "0.3"));
            _cc.VarList.Add(new VariableItem("客户级别", "土豪"));
            _cc.VarUsings.Add(new UsingItem() { Namespace = "System.IO", Path = typeof(Path).Assembly.Location });
            AddUseExpr(_cc);
            var _curCompileCond = GetTemplate();
            txtRunTime.Text = "";
            var _stopWath = new System.Diagnostics.Stopwatch();
            _stopWath.Start();
            CompileList();
            _stopWath.Stop();
            txtRunTime.Text = "编译多个模板时间： " + _stopWath.ElapsedMilliseconds + "毫秒";
            _stopWath.Restart();
            var _intTimes = 0;
            int.TryParse(cmbTimes.Text.Replace(",", ""), out _intTimes);
            CompileResult _result = null;
            for (int i = 0; i < _intTimes; i++)
            {
                _result = ExpressionFacade.Parse(_cc, _curCompileCond);
            }
            _stopWath.Stop();
            txtRunTime.Text += Environment.NewLine + "运行" + _intTimes + "次的时间： " + _stopWath.ElapsedMilliseconds + "毫秒";
            if (_result == null) return;
            if (_result.IsSuccess)
                txtResult.Text = "返回结果：" + _result.Cond;
            else
                txtResult.Text = "发生错误：" + _result.ErrorMsg;
        }

        private void CompileList()
        {
            List<CompileCond> _tempateList = new List<CompileCond>();
            foreach (var item in BinarySerialize.GetConds())
            {
                var _obj = BinarySerialize.Deserialize(item) as CompileCond;
                if (_obj != null)
                {
                    _tempateList.Add(_obj);
                }
            }
            ExpressionFacade.CompileList(_tempateList.ToArray());
        }

        private CompileCond GetTemplate()
        {
            CompileCond _cc = new CompileCond();
            _cc.Name = this.GetType().Name;
            _cc.VarList.Add(new VariableItem("打折", typeof(decimal).FullName, null));
            _cc.VarList.Add(new VariableItem("客户级别", typeof(string).FullName, null));
            _cc.VarTableDef.Add(new VariableItem("物品", typeof(string).FullName, null));
            _cc.VarTableDef.Add(new VariableItem("单价", typeof(decimal).FullName, null));
            _cc.VarTableDef.Add(new VariableItem("数量", typeof(decimal).FullName, null));
            AddUseExpr(_cc);
            _cc.VarUsings.Add(new UsingItem() { Namespace = "System.IO", Path = typeof(Path).Assembly.Location });
            _cc.Context = "_SayHelloWorld(\"小明\",\"大家好\") +  _小明哥哥 + _GetAge(_小明哥哥) + 客户级别 + 打折";
            _cc.Context += "+ List[1].物品 + \"单价\" + List[1].单价 + \"数量\" + List[1].数量 + Directory.GetFiles(\"C:\\\\\")[0]";
            txtExpr.Text = _cc.Context;
            _cc.IsSingle = true;
            return _cc;
        }
    }
}
