﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ExpressionEditor.Sample
{
    public partial class FrmMathSample : Form
    {
        private static string _path;
        private CompileCond _curCompileCond;

        public FrmMathSample()
        {
            InitializeComponent();
        }

        private void FrmMath_Load(object sender, EventArgs e)
        {
            _path = BinarySerialize.GetPath(this.GetType());
            if (File.Exists(_path))
            {
                lblOldDesign.Text = "设计：" + _path;
                _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
                txtResult.Text = "计算公式已经存在，可以直接计算！";
            }
        }

        private void tsbDesign_Click(object sender, EventArgs e)
        {
            CompileCond _cc = _curCompileCond;
            if (_cc == null)
            {
                _cc = new CompileCond();
            }
            _cc.Name = this.GetType().Name;
            _cc.VarList.Clear();
            if (ckbUnitPrice.Checked)
                _cc.VarList.Add(new VariableItem("单价", Convert.ToDouble(txtUnitPrice.Text)));
            if (ckbAmount.Checked)
                _cc.VarList.Add(new VariableItem("数量", Convert.ToDouble(txtAmount.Text)));
            if (ckbDiscount.Checked)
                _cc.VarList.Add(new VariableItem("打折", Convert.ToDouble(txtDiscount.Text)));
            var _obj = ExpressionFacade.LoadDesigner(_cc);
            if (_obj != null)
            {
                ExpressionFacade.RemoveLoadedType(_cc.Name);
                BinarySerialize.Serialize(_obj, _path);
                _curCompileCond = _obj;
                lblOldDesign.Text = "设计：" + _path;
                MessageBox.Show("设计完成！");
            }
        }

        private void tsbExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void tsbCalc_Click(object sender, EventArgs e)
        {
            if (_curCompileCond == null)
            {
                if (File.Exists(_path))
                {
                    _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
                }
            }
            if (_curCompileCond == null)
            {
                MessageBox.Show("还没有公式，请先行设计！");
                return;
            }
            if (_curCompileCond != null)
            {
                CompileCond _cc = new CompileCond();
                _cc.Name = this.GetType().Name;
                if (ckbUnitPrice.Checked)
                    _cc.VarList.Add(new VariableItem("单价", Convert.ToDouble(txtUnitPrice.Text)));
                if (ckbAmount.Checked)
                    _cc.VarList.Add(new VariableItem("数量", Convert.ToDouble(txtAmount.Text)));
                if (ckbDiscount.Checked)
                    _cc.VarList.Add(new VariableItem("打折", Convert.ToDouble(txtDiscount.Text)));
                var _result = ExpressionFacade.Parse(_cc, _curCompileCond);
                if (_result.IsSuccess)
                    txtResult.Text = "返回结果：" + _result.Cond;
                else
                    txtResult.Text = "发生错误：" + _result.ErrorMsg;
            }
        }

        private void tsbDel_Click(object sender, EventArgs e)
        {
            if (File.Exists(_path))
            {
                File.Delete(_path);
                _curCompileCond = null;
                lblOldDesign.Text = "";
                MessageBox.Show("删除成功！");
            }

        }
    }
}
