﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ExpressionEditor.Sample
{
    public partial class FrmNewUsingSample : Form
    {
        private static string _path;
        private CompileCond _curCompileCond;
        private DataTable _table;

        public FrmNewUsingSample()
        {
            InitializeComponent();
        }

        private void FrmMath_Load(object sender, EventArgs e)
        {
            _table = new DataTable();
            _table.Columns.Add("物品");
            _table.Columns.Add("单价", typeof(Decimal));
            _table.Columns.Add("数量", typeof(Decimal));
            _table.Rows.Add("钢笔", 10, 2);
            _table.Rows.Add("硬皮抄", 10, 3);
            _table.Rows.Add("沐浴露", 20, 1);
            //dataGridView1.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            _path = BinarySerialize.GetPath(this.GetType());
            if (File.Exists(_path))
            {
                lblOldDesign.Text = "设计：" + _path;
                _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
                txtResult.Text = "计算公式已经存在，可以直接计算！";
            }
        }

        private void tsbDesign_Click(object sender, EventArgs e)
        {
            CompileCond _cc = _curCompileCond;
            if (_cc == null)
            {
                _cc = new CompileCond();

            }
            _cc.Name = this.GetType().Name;
            _cc.VarUsings.Clear();
            if (ckbUseUsing.Checked)
                _cc.VarUsings.Add(new UsingItem() { Namespace = "System.IO", Path = typeof(Path).Assembly.Location });

            //_cc.VarTable = _table;
            var _obj = ExpressionFacade.LoadDesigner(_cc);
            if (_obj != null)
            {
                ExpressionFacade.RemoveLoadedType(_cc.Name);
                BinarySerialize.Serialize(_obj, _path);
                _curCompileCond = _obj;
                lblOldDesign.Text = "设计：" + _path;
                MessageBox.Show("设计完成！");
            }
        }

        private void tsbExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void tsbCalc_Click(object sender, EventArgs e)
        {
            if (_curCompileCond == null)
            {
                if (File.Exists(_path))
                {
                    _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
                }
            }
            if (_curCompileCond == null)
            {
                MessageBox.Show("还没有公式，请先行设计！");
                return;
            }
            if (_curCompileCond != null)
            {
                CompileCond _cc = new CompileCond();
                _cc.Name = this.GetType().Name;
                if (ckbUseUsing.Checked)
                    _cc.VarUsings.Add(new UsingItem() { Namespace = "System.IO", Path = typeof(Path).Assembly.Location });
                //if (ckbUseUsing.Checked)
                //    _cc.VarUsings.Add(new UsingItem() { Namespace = "System.IO", Path = typeof(Path).Assembly.Location });
                var _result = ExpressionFacade.Parse(_cc, _curCompileCond);
                if (_result.IsSuccess)
                    txtResult.Text = "返回结果：" + _result.Cond;
                else
                    txtResult.Text = "发生错误：" + _result.ErrorMsg;
            }
        }

        private void tsbDel_Click(object sender, EventArgs e)
        {
            if (File.Exists(_path))
            {
                File.Delete(_path);
                _curCompileCond = null;
                lblOldDesign.Text = "";
                MessageBox.Show("删除成功！");
            }

        }
    }
}
