﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace ExpressionEditor.Sample
{
    public partial class FrmUserFuncSample : Form
    {
        private static string _path;
        private CompileCond _curCompileCond;
        private DataTable _table;

        public FrmUserFuncSample()
        {
            InitializeComponent();
        }

        private void FrmMath_Load(object sender, EventArgs e)
        {
            _table = new DataTable();
            _table.Columns.Add("物品");
            _table.Columns.Add("单价", typeof(Decimal));
            _table.Columns.Add("数量", typeof(Decimal));
            _table.Rows.Add("钢笔", 10, 2);
            _table.Rows.Add("硬皮抄", 10, 3);
            _table.Rows.Add("沐浴露", 20, 1);
            //dataGridView1.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            _path = BinarySerialize.GetPath(this.GetType());
            if (File.Exists(_path))
            {
                lblOldDesign.Text = "设计：" + _path;
                _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
                txtResult.Text = "计算公式已经存在，可以直接计算！";
            }
        }

        private void tsbDesign_Click(object sender, EventArgs e)
        {
            CompileCond _cc = _curCompileCond;
            if (_cc == null)
            {
                _cc = new CompileCond();

            }
            _cc.Name = this.GetType().Name;
            _cc.VarUsings.Clear();
            if (ckbUseExpr.Checked)
            {
                AddUseExpr(_cc);
            }

            //_cc.VarTable = _table;
            var _obj = ExpressionFacade.LoadDesigner(_cc);
            if (_obj != null)
            {
                ExpressionFacade.RemoveLoadedType(_cc.Name);
                BinarySerialize.Serialize(_obj, _path);
                _curCompileCond = _obj;
                lblOldDesign.Text = "设计：" + _path;
                MessageBox.Show("设计完成！");
            }
        }

        private void AddUseExpr(CompileCond cc)
        {
            cc.VarUserExprs.Clear();
            cc.VarUserExprs.Add(new UserExpr()
            {
                Catalog = "Self",
                Syntax = "_SayHelloWorld(string name, string world)",
                Description = "一个给大家Say Hello 的方法",
                Example = "string _str = _SayHelloWorld(\"张三\",\"大家好\")",
                Name = "_SayHelloWorld",
                Function = new Func<string, string, string>(this._SayHelloWorld),
            });            
            if (ckbStrFunc.Checked)
            {
                cc.VarUserExprs.Add(new UserExpr()
                {
                    Catalog = "Self",
                    Syntax = "_小明哥哥",
                    Description = "小明哥哥属性",
                    Example = "string _str = _小明哥哥",
                    Name = "_小明哥哥",
                    ReturnType = typeof(string).FullName,
                    Content = "\"大明\" + DateTime.Now.ToString(\"MMdd\")",
                    IsMethod = false
                });
                cc.VarUserExprs.Add(new UserExpr()
                {
                    Catalog = "Self",
                    Syntax = "_GetAge(string name)",
                    Description = "获取年龄",
                    Example = "int _age = _GetAge(\"张三\")",
                    Name = "_GetAge(string name)",
                    ReturnType = typeof(int).FullName,
                    Content = "return DateTime.Now.Year + 18;",
                    IsMethod = true
                });
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="name"></param>
        /// <param name="world"></param>
        /// <returns></returns>
        private string _SayHelloWorld(string name, string world)
        {
            return name + "讲 : " + world + " 当前时间为： " + DateTime.Now;
        }

        private void tsbExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void tsbCalc_Click(object sender, EventArgs e)
        {
            if (_curCompileCond == null)
            {
                if (File.Exists(_path))
                {
                    _curCompileCond = BinarySerialize.Deserialize(_path) as CompileCond;
                }
            }
            if (_curCompileCond == null)
            {
                MessageBox.Show("还没有公式，请先行设计！");
                return;
            }
            if (_curCompileCond != null)
            {
                CompileCond _cc = new CompileCond();
                _cc.Name = this.GetType().Name;
                if (ckbUseExpr.Checked)
                {
                    AddUseExpr(_cc);
                }
                //if (ckbUseUsing.Checked)
                //    _cc.VarUsings.Add(new UsingItem() { Namespace = "System.IO", Path = typeof(Path).Assembly.Location });
                var _result = ExpressionFacade.Parse(_cc, _curCompileCond);
                if (_result.IsSuccess)
                    txtResult.Text = "返回结果：" + _result.Cond;
                else
                    txtResult.Text = "发生错误：" + _result.ErrorMsg;
            }
        }

        private void tsbDel_Click(object sender, EventArgs e)
        {
            if (File.Exists(_path))
            {
                File.Delete(_path);
                _curCompileCond = null;
                lblOldDesign.Text = "";
                MessageBox.Show("删除成功！");
            }

        }
    }
}
